import logging

from .coap import CoapMessage

LOGGER = logging.getLogger(__name__)


class MomitCool:
    """Represents a single MomitCool device."""

    def __init__(self, host: str, port: int=19924):
        """Initializes a new instance of the :see:MomitCool class
        with the specified host and port."""

        self.host = host
        self.port = port

    def on(self):
        """Turns on the air-conditioning."""

        LOGGER.info('Turning on air-conditioning')

        params = (
            ('cm', 'cool'), # mode: 'cool' or 'heat'
            ('t', '1440'), # time till turn off in minutes, 24 hours
            ('sp', '0'), # target temperature, 290 = 29.0C
        )

        message = CoapMessage(host=self.host, port=self.port)
        message.options.path = ['10242', '0', '0']
        message.options.content_format = 1541
        message.payload = self._encode_parameters(params)
        message.send()

    def off(self):
        """Turns off the air-conditioning."""

        LOGGER.info('Turning off air-conditioning')

        params = (
            ('cm', 'cool'),
            ('t', '0'),
            ('sp', 'off')
        )

        message = CoapMessage(host=self.host, port=self.port)
        message.options.path = ['10242', '0', '0']
        message.options.content_format = 1541
        message.payload = self._encode_parameters(params)
        message.send()

    def _encode_parameters(self, parameters: dict):
        """Encodes the payload parameters for transmission."""

        result = ','.join([
            '%s=%s' % (key, value)
            for key, value in parameters
        ])

        return result.encode()



import sys
import socket
import struct
import logging

from enum import Enum
from random import randint
from typing import List

LOGGER = logging.getLogger(__name__)

COAP_DEFAULT_PORT = 5683


class CoapCode(Enum):
    """Enumuration of available CoAP codes."""

    PUT = 3


class CoapOptionCode(Enum):
    """Enumuration of codes for CoAP options."""

    URI_PATH = 0xb5
    URI_PATH_SEPARATOR = 0x01
    CONTENT_FORMAT = 0x12
    END = 0xff


class CoapOptions:
    """CoAP request options."""

    path = []
    content_format = None

    def __init__(self, **kwargs):
        """Initializes a new instance of the :see:CoapOptions
        class with the specified set of options."""

        for key, value in kwargs.items():
            setattr(self, key, value)

    def encode(self):
        """Encodes this set of options."""

        options_buffer = b''

        if self.path and len(self.path) > 0:
            options_buffer += struct.pack('B', CoapOptionCode.URI_PATH.value)
            for index, path in enumerate(self.path):
                options_buffer += path.encode()

                # no trailing separator
                if index != len(self.path) - 1:
                    options_buffer += struct.pack('B', CoapOptionCode.URI_PATH_SEPARATOR.value)

        if self.content_format:
            options_buffer += struct.pack('B', CoapOptionCode.CONTENT_FORMAT.value)
            options_buffer += struct.pack('!H', self.content_format)

        options_buffer += struct.pack('B', CoapOptionCode.END.value)
        return options_buffer


class CoapMessage:
    """A CoAP message."""

    host = 'localhost'
    port = COAP_DEFAULT_PORT
    code = CoapCode.PUT
    options = CoapOptions()
    payload = b''

    def __init__(self, **kwargs):
        """Initializes a new instance of the :see:CoapMessage class."""

        for key, value in kwargs.items():
            setattr(self, key, value)

    def encode(self):
        """Encodes the message into a binary structure that can
        be send over the wire."""

        message_id = randint(0x0, 0xffff) # maximum value of ushrt
        token = struct.pack('I', randint(0, 0xffffffff)) # maxmium value of uint

        request_buffer = b'\x44'
        request_buffer += struct.pack('B', self.code.value)
        request_buffer += struct.pack('H', randint(0x0, message_id))
        request_buffer += token
        request_buffer += self.options.encode()
        request_buffer += self.payload

        return request_buffer

    def send(self):
        """Sends the message to the configured destination."""

        packet = self.encode()
        LOGGER.debug('%s coap://%s:%s/%s (%d bytes)\n\n' \
                     '%s',
                     self.code.name, self.host, self.port,
                     '/'.join(self.options.path), len(packet),
                     str(self.payload))

        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.bind(('0.0.0.0', COAP_DEFAULT_PORT))
        sock.sendto(packet, (self.host, self.port))



import os

from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

setup(
    name='py-momit-cool-remote',
    version='1.0',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='Library for remotely interfacing with the Momit Cool air conditioning controller.',
    long_description=README,
    url='https://github.com/Photonios/py-momit-cool-remote',
    author='Swen Kooij',
    author_email='swenkooij@gmail.com',
    keywords=['momit', 'cool', 'remote', 'control'],
    entry_points={
        'console_scripts': [
            'momit-cool=cli:main'
        ]
    },
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ]
)
